@@ -6,6 +6,7 @@ import builtins
 
 from xonsh.lazyasd import lazyobject
 
+
 def _splitpath(path, sofar=[]):
     folder, path = os.path.split(path)
     if path == "":
@@ -35,6 +36,7 @@ def _make_if_not_exists(dirname):
     if not os.path.isdir(dirname):
         os.makedirs(dirname)
 
+
 def should_use_cache(execer, mode):
     """
     Return ``True`` if caching has been enabled for this mode (through command
@@ -42,7 +44,7 @@ def should_use_cache(execer, mode):
     """
     if mode == 'exec':
         return ((execer.scriptcache or
-                    execer.cacheall) and
+                 execer.cacheall) and
                 (builtins.__xonsh_env__['XONSH_CACHE_SCRIPTS'] or
                     builtins.__xonsh_env__['XONSH_CACHE_EVERYTHING']))
     else:
@@ -99,7 +101,8 @@ def compile_code(filename, code, execer, glb, loc, mode):
             code += '\n'
         old_filename = execer.filename
         execer.filename = filename
-        ccode = execer.compile(code, glbs=glb, locs=loc, mode=mode)
+        ccode = execer.compile(code, glbs=glb, locs=loc, mode=mode,
+                               filename=filename)
     except Exception:
         raise
     finally:
@@ -132,8 +135,8 @@ def run_script_with_cache(filename, execer, glb=None, loc=None, mode='exec'):
     """
     run_cached = False
     use_cache = should_use_cache(execer, mode)
+    cachefname = get_cache_filename(filename, code=False)
     if use_cache:
-        cachefname = get_cache_filename(filename, code=False)
         run_cached, ccode = script_cache_check(filename, cachefname)
     if not run_cached:
         with open(filename, 'r') as f:
