@@ -16,7 +16,7 @@ class Execer(object):
     """Executes xonsh code in a context."""
 
     def __init__(self, filename='<xonsh-code>', debug_level=0, parser_args=None,
-                 unload=True, config=None, login=True, xonsh_ctx=None):
+                 unload=True, xonsh_ctx=None, scriptcache=True, cacheall=False):
         """Parameters
         ----------
         filename : str, optional
@@ -27,18 +27,24 @@ class Execer(object):
             Arguments to pass down to the parser.
         unload : bool, optional
             Whether or not to unload xonsh builtins upon deletion.
-        config : str, optional
-            Path to configuration file.
         xonsh_ctx : dict or None, optional
             Xonsh xontext to load as builtins.__xonsh_ctx__
+        scriptcache : bool, optional
+            Whether or not to use a precompiled bytecode cache when execing
+            code, default: True.
+        cacheall : bool, optional
+            Whether or not to cache all xonsh code, and not just files. If this
+            is set to true, it will cache command line input too, default: False.
         """
         parser_args = parser_args or {}
         self.parser = Parser(**parser_args)
         self.filename = filename
         self.debug_level = debug_level
         self.unload = unload
+        self.scriptcache = scriptcache
+        self.cacheall = cacheall
         self.ctxtransformer = CtxAwareTransformer(self.parser)
-        load_builtins(execer=self, config=config, login=login, ctx=xonsh_ctx)
+        load_builtins(execer=self, ctx=xonsh_ctx)
 
     def __del__(self):
         if self.unload:
