@@ -4,8 +4,10 @@ on a platform.
 """
 import os
 import sys
+import ctypes
 import signal
 import pathlib
+import builtins
 import platform
 import functools
 import subprocess
@@ -50,9 +52,18 @@ ON_FREEBSD = LazyBool(lambda: (sys.platform.startswith('freebsd')),
 ON_NETBSD = LazyBool(lambda: (sys.platform.startswith('netbsd')),
                      globals(), 'ON_NETBSD')
 """``True`` if on a NetBSD operating system, else ``False``."""
-ON_BSD = LazyBool(lambda: ON_FREEBSD or ON_NETBSD,
-                  globals(), 'ON_BSD')
-"""``True`` if on a BSD operating system, else ``False``."""
+
+
+@lazybool
+def ON_BSD():
+    """``True`` if on a BSD operating system, else ``False``."""
+    return bool(ON_FREEBSD) or bool(ON_NETBSD)
+
+
+@lazybool
+def ON_BEOS():
+    """True if we are on BeOS or Haiku."""
+    return sys.platform == 'beos5' or sys.platform == 'haiku1'
 
 
 #
@@ -165,6 +176,15 @@ def pathbasename(p):
     """
     return pathsplit(p)[-1]
 
+# termios tc(get|set)attr indexes.
+IFLAG = 0
+OFLAG = 1
+CFLAG = 2
+LFLAG = 3
+ISPEED = 4
+OSPEED = 5
+CC = 6
+
 
 #
 # Dev release info
@@ -174,12 +194,15 @@ def pathbasename(p):
 def githash():
     """Returns a tuple contains two strings: the hash and the date."""
     install_base = os.path.dirname(__file__)
+    githash_file = '{}/dev.githash'.format(install_base)
+    if not os.path.exists(githash_file):
+        return None, None
     sha = None
     date_ = None
     try:
-        with open('{}/dev.githash'.format(install_base), 'r') as f:
+        with open(githash_file) as f:
             sha, date_ = f.read().strip().split('|')
-    except FileNotFoundError:
+    except ValueError:
         pass
     return sha, date_
 
@@ -273,16 +296,22 @@ def windows_bash_command():
     # Check that bash is on path otherwise try the default directory
     # used by Git for windows
     wbc = 'bash'
-    try:
-        subprocess.check_call([wbc, '--version'],
-                              stdout=subprocess.PIPE,
-                              stderr=subprocess.PIPE)
-    except (FileNotFoundError, subprocess.CalledProcessError):
-        gfwp = git_for_windows_path()
-        if gfwp:
-            bashcmd = os.path.join(gfwp, 'bin\\bash.exe')
-            if os.path.isfile(bashcmd):
-                wbc = bashcmd
+    bash_on_path = builtins.__xonsh_commands_cache__.lazy_locate_binary('bash',
+                                                                        ignore_alias=True)
+    if bash_on_path:
+        # Check if Bash is from the "Windows Subsystem for Linux" (WSL)
+        # which can't be used by xonsh foreign-shell/completer
+        out = subprocess.check_output([bash_on_path, '--version'],
+                                      stderr=subprocess.PIPE,
+                                      universal_newlines=True)
+        if 'pc-linux-gnu' in out.splitlines()[0]:
+            gfwp = git_for_windows_path()
+            if gfwp:
+                bashcmd = os.path.join(gfwp, 'bin\\bash.exe')
+                if os.path.isfile(bashcmd):
+                    wbc = bashcmd
+        else:
+            wbc = bash_on_path
     return wbc
 
 #
@@ -342,3 +371,60 @@ def PATH_DEFAULT():
     else:
         pd = ()
     return pd
+
+
+#
+# libc
+#
+@lazyobject
+def LIBC():
+    """The platform dependent libc implementation."""
+    if ON_DARWIN:
+        libc = ctypes.CDLL(ctypes.util.find_library("c"))
+    elif ON_CYGWIN:
+        libc = ctypes.CDLL('cygwin1.dll')
+    elif ON_BSD:
+        try:
+            libc = ctypes.CDLL('libc.so')
+        except AttributeError:
+            libc = None
+        except OSError:
+            # OS X; can't use ctypes.util.find_library because that creates
+            # a new process on Linux, which is undesirable.
+            try:
+                libc = ctypes.CDLL('libc.dylib')
+            except OSError:
+                libc = None
+    elif ON_POSIX:
+        try:
+            libc = ctypes.CDLL('libc.so')
+        except AttributeError:
+            libc = None
+        except OSError:
+            # Debian and derivatives do the wrong thing because /usr/lib/libc.so
+            # is a GNU ld script rather than an ELF object. To get around this, we
+            # have to be more specific.
+            # We don't want to use ctypes.util.find_library because that creates a
+            # new process on Linux. We also don't want to try too hard because at
+            # this point we're already pretty sure this isn't Linux.
+            try:
+                libc = ctypes.CDLL('libc.so.6')
+            except OSError:
+                libc = None
+        if not hasattr(libc, 'sysinfo'):
+            # Not Linux.
+            libc = None
+    elif ON_WINDOWS:
+        if hasattr(ctypes, 'windll') and hasattr(ctypes.windll, 'kernel32'):
+            libc = ctypes.windll.kernel32
+        else:
+            try:
+                # Windows CE uses the cdecl calling convention.
+                libc = ctypes.CDLL('coredll.lib')
+            except (AttributeError, OSError):
+                libc = None
+    elif ON_BEOS:
+        libc = ctypes.CDLL('libroot.so')
+    else:
+        libc = None
+    return libc
