@@ -7,6 +7,7 @@ The following time_it alias and Timer was forked from the IPython project:
 * Copyright (c) 2001, Janko Hauser <jhauser@zscout.de>
 * Copyright (c) 2001, Nathaniel Gray <n8gray@caltech.edu>
 """
+import os
 import gc
 import sys
 import math
@@ -16,6 +17,8 @@ import builtins
 import itertools
 
 from xonsh.lazyasd import lazyobject, lazybool
+from xonsh.events import events
+from xonsh.platform import ON_WINDOWS
 
 
 @lazybool
@@ -223,3 +226,92 @@ def timeit_alias(args, stdin=None):
         if tc > tc_min:
             print("Compiler time: {0:.2f} s".format(tc))
     return
+
+
+_timings = {'start': clock() if ON_WINDOWS else 0.0}
+
+
+def setup_timings():
+    global _timings
+    if '--timings' in sys.argv:
+        events.doc('on_timingprobe', """
+        on_timingprobe(name: str) -> None
+
+        Fired to insert some timings into the startuptime list
+        """)
+
+        @events.on_timingprobe
+        def timing_on_timingprobe(name, **kw):
+            global _timings
+            _timings[name] = clock()
+
+        @events.on_post_cmdloop
+        def timing_on_post_cmdloop(**kw):
+            global _timings
+            _timings['on_post_cmdloop'] = clock()
+
+        @events.on_post_init
+        def timing_on_post_init(**kw):
+            global _timings
+            _timings['on_post_init'] = clock()
+
+        @events.on_post_rc
+        def timing_on_post_rc(**kw):
+            global _timings
+            _timings['on_post_rc'] = clock()
+
+        @events.on_postcommand
+        def timing_on_postcommand(**kw):
+            global _timings
+            _timings['on_postcommand'] = clock()
+
+        @events.on_pre_cmdloop
+        def timing_on_pre_cmdloop(**kw):
+            global _timings
+            _timings['on_pre_cmdloop'] = clock()
+
+        @events.on_pre_rc
+        def timing_on_pre_rc(**kw):
+            global _timings
+            _timings['on_pre_rc'] = clock()
+
+        @events.on_precommand
+        def timing_on_precommand(**kw):
+            global _timings
+            _timings['on_precommand'] = clock()
+
+        @events.on_ptk_create
+        def timing_on_ptk_create(**kw):
+            global _timings
+            _timings['on_ptk_create'] = clock()
+
+        @events.on_chdir
+        def timing_on_chdir(**kw):
+            global _timings
+            _timings['on_chdir'] = clock()
+
+        @events.on_post_prompt
+        def timing_on_post_prompt(**kw):
+            global _timings
+            _timings = {'on_post_prompt': clock()}
+
+        @events.on_pre_prompt
+        def timing_on_pre_prompt(**kw):
+            global _timings
+            _timings['on_pre_prompt'] = clock()
+            times = list(_timings.items())
+            times = sorted(times, key=lambda x: x[1])
+            width = max(len(s) for s, _ in times) + 2
+            header_format = '|{{:<{}}}|{{:^11}}|{{:^11}}|'.format(width)
+            entry_format = '|{{:<{}}}|{{:^11.3f}}|{{:^11.3f}}|'.format(width)
+            sepline = '|{}|{}|{}|'.format('-'*width, '-'*11, '-'*11)
+            # Print result table
+            print(' Debug level: {}'.format(os.getenv('XONSH_DEBUG', 'Off')))
+            print(sepline)
+            print(header_format.format('Event name', 'Time (s)', 'Delta (s)'))
+            print(sepline)
+            prevtime = tstart = times[0][1]
+            for name, ts in times:
+                print(entry_format.format(name, ts - tstart, ts - prevtime))
+                prevtime = ts
+            print(sepline)
