@@ -792,8 +792,11 @@ def on_main_thread():
     return threading.current_thread() is threading.main_thread()
 
 
+_DEFAULT_SENTINEL = object()
+
+
 @contextlib.contextmanager
-def swap(namespace, name, value, default=NotImplemented):
+def swap(namespace, name, value, default=_DEFAULT_SENTINEL):
     """Swaps a current variable name in a namespace for another value, and then
     replaces it when the context is exited.
     """
@@ -805,6 +808,21 @@ def swap(namespace, name, value, default=NotImplemented):
     else:
         setattr(namespace, name, old)
 
+
+@contextlib.contextmanager
+def swap_values(d, updates, default=_DEFAULT_SENTINEL):
+    """Updates a dictionary (or other mapping) with values from another mapping,
+    and then restores the original mapping when the context is exited.
+    """
+    old = {k: d.get(k, default) for k in updates}
+    d.update(updates)
+    yield
+    for k, v in old.items():
+        if v is default and k in d:
+            del d[k]
+        else:
+            d[k] = v
+
 #
 # Validators and converters
 #
@@ -1773,3 +1791,21 @@ def columnize(elems, width=80, newline='\n'):
     lines = [row_t.format(row=row, w=colwidths) for row in
              itertools.zip_longest(*data, fillvalue='')]
     return lines
+
+
+def unthreadable(f):
+    """Decorator that specifies that a callable alias should be run only
+    on the main thread process. This is often needed for debuggers and
+    profilers.
+    """
+    f.__xonsh_threadable__ = False
+    return f
+
+
+def uncapturable(f):
+    """Decorator that specifies that a callable alias should not be run with
+    any capturing. This is often needed if the alias call interactive
+    subprocess, like pagers and text editors.
+    """
+    f.__xonsh_capturable__ = False
+    return f
