@@ -1,12 +1,15 @@
 """Matplotlib xontribution."""
 
-from xonsh.proc import foreground as _foreground
+from xonsh.tools import unthreadable
 
-@_foreground
-def _mpl(args, stdin=None):
+__all__ = ()
+
+
+@unthreadable
+def mpl(args, stdin=None):
     """Hooks to matplotlib"""
     from xontrib.mplhooks import show
     show()
 
 
-aliases['mpl'] = _mpl
+aliases['mpl'] = mpl
